//  ------------------------------------------------------------------------
//@ PanelBmp.pas
//  TPanelBmp VCL Component
//  Copyright (c) Cole Joe Taiwan 1998 All rights reserved.
//  email: kchou@adc.iii.org.tw
//  -   Description
//      TPanelBmp is control of TPanel attached a image on it
//  ------------------------------------------------------------------------
unit PanelBmp;
//BtnAni
//ImageAni
//MyGridImg
//BmpAni
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type
  TPanelBmp = class(TPanel)
  private
    { Private declarations }
    m_sFileName: string;
    m_Image: TImage;
    m_bOnPaint: boolean;
  protected
    { Protected declarations }
    function    IsBitmapEmpty: boolean;
    function    GetPicture: TPicture;
    procedure   SetPicture(Value: TPicture);
    function    GetAutoSize: boolean;
    procedure   SetAutoSize(Value: boolean);
    procedure   LoadFromFile(Value: string);
    //procedure   Paint; override;
    procedure   WMPaint(var Message: TWMPaint); message WM_PAINT;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
  published
    { Published declarations }
    property AutoSize: Boolean              read GetAutoSize
                                            write SetAutoSize;
    property Picture:  TPicture             read  GetPicture
                                            write SetPicture;
    property FileName: string               read m_sFileName
                                            write LoadFromFile;
  end;

implementation
{$R PanelBmp.res}
constructor TPanelBmp.Create(AOwner: TComponent);
begin // [
  	inherited Create(AOwner);
  	//ControlStyle:=ControlStyle + [csReplicatable];
    m_bOnPaint:=FALSE;
    m_sFileName:='';
    Width:=30;
    Height:=30;
    m_Image:=TImage.Create(self);
    m_Image.Parent:=self;
    m_Image.Align:=alClient;
    m_Image.AutoSize:=TRUE;
    m_Image.OnClick:=OnClick;
    m_Image.OnDblClick:=OnDblClick;
    m_Image.OnStartDrag:=OnStartDrag;
    m_Image.OnDragDrop:=OnDragDrop;
    m_Image.OnDragOver:=OnDragOver;
    m_Image.OnEndDrag:=OnEndDrag;
    m_Image.OnMouseDown:=OnMouseDown;
    m_Image.OnMouseMove:=OnMouseMove;
    m_Image.OnMouseUp:=OnMouseUp;
end; // ] TPanelBmp.Create
function TPanelBmp.IsBitmapEmpty: boolean;
begin
    Result:=m_Image.Picture.Bitmap.Empty;
end;
destructor TPanelBmp.Destroy;
begin // [
    m_Image.Free;
  	inherited Destroy;
end; // ] TPanelBmp.Destroy
function TPanelBmp.GetPicture: TPicture;
begin
    Result:=m_Image.Picture;
end;
procedure TPanelBmp.SetPicture(Value: TPicture);
begin // [
  	m_Image.Picture.Assign(Value);
    Invalidate;
end; // ] TPanelBmp.SetPicture
procedure TPanelBmp.LoadFromFile(Value: string);
begin //[
    if  not FileExists(Value) then begin
        Picture:=nil; m_sFileName:=''; exit;
    end;
    if  CompareText(Value, m_sFileName)=0 then exit; // EqualNoCase
    m_sFileName:=Value;
    try // [
        m_Image.Picture.LoadFromFile(Value);
    except
        m_sFileName:='';
        raise;
    end; // ] try...except
    Invalidate;
end; // ] TPanelBmp.LoadFromFile
function TPanelBmp.GetAutoSize: boolean;
begin //[
    Result:=m_Image.AutoSize;
end; // ] TPanelBmp.GetAutoSize
procedure TPanelBmp.SetAutoSize(Value: boolean);
begin //[
    if  Value=m_Image.AutoSize then exit;
    m_Image.AutoSize:=Value;
    m_Image.Stretch:=not m_Image.AutoSize;
    Invalidate;
end; // ] TPanelBmp.SetAutoSize
//procedure TPanelBmp.Paint;
//  ------------------------------------------------------------------------
//@ TPanelBmp.WMPaint
//  procedure TPanelBmp.WMPaint(var Message: TWMPaint);
//  Since TPanelBmp.Paint WON'T repint object under the following noted condition,
//  we implement invalidate by the way of capture WM_PAINT message
//  -   Note:
//      if both BevelInner and BevelOuter are bvNone Paint method won't
//      be invoked by its parent TWinControl.WMPaint
//  ------------------------------------------------------------------------
procedure TPanelBmp.WMPaint(var Message: TWMPaint);
var
    nWidth, nHeight: integer;
begin // [
    if  m_bOnPaint then exit;
    m_bOnPaint:=TRUE;
    try // [
        inherited;
        if  AutoSize then begin
            if  not IsBitmapEmpty then begin
                (***
                ClientWidth:=m_Image.Picture.Width;
                ClientHeight:=m_Image.Picture.Height;
                ***)
                //////////////////////////////////
                nWidth:=m_Image.Picture.Width+BorderWidth*2;
                nHeight:=m_Image.Picture.Height+BorderWidth*2;
                if  BevelInner<>bvNone then begin
                    inc(nWidth, BevelWidth*2);
                    inc(nHeight, BevelWidth*2);
                end;
                if  BevelOuter<>bvNone then begin
                    inc(nWidth, BevelWidth*2);
                    inc(nHeight, BevelWidth*2);
                end;
                Width:=nWidth; Height:=nHeight;
            end;
        end;
    finally
        //inherited Invalidate;
        //inherited Paint;
        m_bOnPaint:=FALSE;
    end; // ] try...finally
end; // ] TPanelBmp.Paint

end.
